/*****************************************************************************

	HelloLocale.cpp
	
	Copyright 2001, Siegfried Soft.   All Rights Reserved.

*****************************************************************************/
#include <Alert.h>
#include <Application.h>
#include <PropertyInfo.h>
#include <Roster.h>
#include <String.h>
#include "HelloLocale.h"
#include "HelloLocaleWin.h"
#include "SFTexts.h"

/*****************************************************************************
Here we go!
*****************************************************************************/
int main()
{
HelloLocaleApp	*myApplication;

myApplication = new HelloLocaleApp();		// create/intialize application
myApplication->Run();						// start application

delete(myApplication);						// terminate application
return(0);									// see you...
}

/*****************************************************************************
HelloLocaleApp	Constructor function for the application. The initialization 
			of the application is done here

Usage:		HelloLocaleApp *app;
			app = new HelloLocaleApp();

Parameter:	-
			
Returns:	-

Comments:	- 
*****************************************************************************/
HelloLocaleApp::HelloLocaleApp()		
	: BApplication("application/x-vnd.siegfriedsoft-hellolocale")
{
HelloLocaleWin	*win;		

//----- init Siegfried Locale Library ----------------------------------------
cLang = SFLocale::GetInstance();							// set instance to locale library
cLang->SetAppID(SF_TEXT_APP_ID);							// set ID for "locale" files
cLang->SetDefaultText(mText, SF_LAST_ID, SF_DEFAULT_LANGUAGE);	// set default text and native language name

//----- create/open application window --------------------------------------- 
win = new HelloLocaleWin("Hello Locale");					// create window
win->Show();												// display window
}

/*****************************************************************************
AboutRequested	Hook function; called by the OS to show the "About this 
			famous application" window.

Usage:		-

Parameter:	-

Returns:	-

Comment:	-
*****************************************************************************/
void HelloLocaleApp::AboutRequested(void)
{
BAlert	*Info;
BString	text;

text << "HELLO LOCALE\n\n";
text += cLang->Text(SFINFOTEXT);
text += "\n\n";
text += "Copyright 2000-2001 by Siegfried Soft\n\n";
text += "Siegfried Soft\n";
text += "Hauff-Weingärtner GbR\n";
text += "Software Development\n";
text += "Reichenberger Str. 12\n";
text += "D-34246 Vellmar\nGERMANY\n\n";
text += "E-Mail: hauff@siegfried-soft.de\n";
text += "Internet: www.siegfried-soft.de";
Info = new BAlert("About...", text.String(), "OK");
Info->Go();
}

/*****************************************************************************
MessageReceived	Hook function; called by the OS if a message receive the 
			application.

Usage:		-

Parameter:	message		Pointer to BMessage

Returns:	-

Comments:	-
*****************************************************************************/
void HelloLocaleApp::MessageReceived(BMessage *msg)
{
BMessage	spec, reply;
bool		found;
int32		index, what;
const char	*prop;

switch (msg->what)
	{
	//--------- Scripting ---------------------------------------------------	
	case B_GET_PROPERTY:
		found = false;
		if (msg->GetCurrentSpecifier(&index, &spec, &what, &prop) == B_OK)
			{
			reply.what = B_REPLY;
			if (strcmp(prop, "DefaultLanguage") == 0 && what == B_DIRECT_SPECIFIER)
				{
				found = true;
				reply.AddString("result", SF_DEFAULT_LANGUAGE);
				}
			if (strcmp(prop, "DefaultText") == 0 && what == B_DIRECT_SPECIFIER)
				{
				found = true;
				for (index = 0; index < SF_LAST_ID; index++)
					reply.AddString("result", mText[index]);
				}
			if (strcmp(prop, "TextAppID") == 0 && what == B_DIRECT_SPECIFIER)
				{
				found = true;
				reply.AddString("result", SF_TEXT_APP_ID);
				}
			}  							
		if (found)
			msg->SendReply(&reply);
		else
			BApplication::MessageReceived(msg);
		break;
	//-------- Default message management ------------------------------------
	default:
		BApplication::MessageReceived(msg);
	}
}



/*****************************************************************************
GetSupportedSuites	Hook function; called by the OS to get the kind of 
			scripting support

Usage:		-

Parameter:	msg		Pointer to BMessage to add the user defined "suites".

Returns:	-

Comment:	-
*****************************************************************************/
static property_info mPropList[] = {
	{ "DefaultText", {B_GET_PROPERTY, 0}, {B_DIRECT_SPECIFIER, 0}, "get the default texts for localization", 0},
	{ "DefaultLanguage", {B_GET_PROPERTY, 0}, {B_DIRECT_SPECIFIER, 0}, "get the default language name for localization", 0},
	{ "TextAppID", {B_GET_PROPERTY, 0}, {B_DIRECT_SPECIFIER, 0}, "get the id for locale files", 0},
	0 // terminate list
};

status_t HelloLocaleApp::GetSupportedSuites(BMessage *msg)
{
BPropertyInfo PropInfo(mPropList);

msg->AddString("suites", "suite/vnd.SiegfriedSoft-locale");
msg->AddFlat("messages", &PropInfo);

return BApplication::GetSupportedSuites(msg);
}

/*****************************************************************************
ResolveSpecifier	Hook fuction; called by the OS to determine which handler
			is to use for the scripting messsage

Usage:		-

Parameter:	-

Returns:	-

Comments:	-
*****************************************************************************/
BHandler *HelloLocaleApp::ResolveSpecifier(BMessage *msg, int32 index, BMessage *spec, int32 form, const char *prop)
{
BPropertyInfo PropInfo(mPropList);

if (PropInfo.FindMatch(msg, index, spec, form, prop) >= 0)
	return this;

return BApplication::ResolveSpecifier(msg, index, spec, form, prop);
}   
