/*****************************************************************************

	HelloLocaleWin.cpp
	
	Copyright 2001, Siegfried Soft.   All Rights Reserved.

*****************************************************************************/
#include <stdio.h>
#include <Alert.h>
#include <Application.h>
#include <Button.h>
#include <File.h>
#include <MenuBar.h>
#include <MenuField.h>
#include <MenuItem.h>
#include <Message.h>
#include <NodeInfo.h>
#include <NodeMonitor.h>
#include <Path.h>
#include <Roster.h>
#include <StringView.h>
#include "HelloLocaleWin.h"
#include "SFTextIDs.h"

#define	BUTTON_DX	100
#define	BUTTON_DY	25

#define HL_LEFT		50
#define HL_TOP		50
#define HL_WIDTH	400
#define HL_HEIGHT	300

/*****************************************************************************
Routines for class HelloLocaleWin
*****************************************************************************/

/*****************************************************************************
HelloLocaleWin	Contructor; Creates the basic window of HelloLocale.

Usage:		HelloLocaleWin	*win;
			const char		*title = "Hello Locale App"
			window_look		look = ...;
			window_feel		feel = ...;
			int32			flags = ...;
			win = new SFLEWindow(title, look, feel, flags);

Parameter:	title		Title of the window
			look		Window look (see BWindow class)
			feel		Window feel (see BWindow class)
			flags		Window control flags (see BWindow class)

Returns:	win			Pointer to instance of the window class

Comment:	-
*****************************************************************************/
HelloLocaleWin::HelloLocaleWin(const char *title, window_look look, window_feel feel, uint32 flags)
				: BWindow(BRect(HL_LEFT, HL_TOP, HL_LEFT+HL_WIDTH, HL_TOP+HL_HEIGHT), title, look, feel, flags)
{
BRect			frame, rect;
BMenuBar		*menubar;
BMenu			*menu, *smenu;
BMenuItem		*menuitem;
BString			string;
app_info		info; 
BPath			path; 
BEntry			entry;
node_ref		nref;
float			left, right, top, dummy, max_x, max_y;
BButton			*button;
BStringView		*strview;
BFile			file;

//----- Language Initialization -----------------------------------------
cLang = SFLocale::GetInstance();			// Get text data access

//----- Set minimal width and height of the window ----------------------
GetSizeLimits(&dummy, &max_x, &dummy, &max_y);
SetSizeLimits(HL_WIDTH, max_x, HL_HEIGHT, max_y);

//----- Activate node watching for "locale" folder ----------------------
be_app->GetAppInfo(&info); 								// get program info
entry.SetTo(&info.ref); 								// get access to program
entry.GetPath(&path); 									// path + filename
path.GetParent(&path);									// only application path is needed
path.Append("locale");									// append language folder

if (entry.SetTo(path.Path()) == B_OK)					// get access to the folder
	{
	entry.GetNodeRef(&nref);							// get node_ref for folder
	watch_node(&nref, B_WATCH_DIRECTORY, this);			// Activate node watching
	}
entry.Unset();

//----- Load Preferences --------------------------------------------------
path.GetParent(&path);									// only application path is needed
path.Append("HelloLocale.pref");						// Append filename for settings
file.SetTo(path.Path(), B_READ_ONLY);
if (file.InitCheck() == B_OK)
	cPref.Unflatten(&file);
if (!cPref.HasString("language"))
	cPref.AddString("language", SF_DEFAULT_LANGUAGE);

//----- Set Language -----------------------------------------------------
path.GetParent(&path);									// only application path is needed
path.Append("locale");									// append language folder
cLang->SetLanguage(&path, cPref.FindString("language"));	// Set last used language
 
//----- Create menu -------------------------------------------------------
frame.Set(0, 0, 1000, 15);
menubar = new BMenuBar(frame, "MB");
	
//----- File menu ---------------------------------------------------------
menu = new BMenu(cLang->Text(SFFILE));

string.SetTo(cLang->Text(SFABOUT));
string += "...";
menuitem = new BMenuItem(string.String(), new BMessage(B_ABOUT_REQUESTED));
menuitem->SetTarget(be_app);
menu->AddItem(menuitem);

menu->AddItem(new BSeparatorItem());

menuitem = new BMenuItem(cLang->Text(SFQUIT), new BMessage(B_QUIT_REQUESTED),'Q',B_COMMAND_KEY);
menuitem->SetTarget(be_app);
menu->AddItem(menuitem);

menubar->AddItem(menu);

//----- Preferences menu ----------------------------------------------------
menu = new BMenu(cLang->Text(SFPREFERENCES));

smenu = new BMenu(cLang->Text(SFLANGUAGE));			// create language menu
BuildLanguageMenu(smenu, cPref.FindString("language"));	// create items for menu
cLocaleMenu = smenu;								// Save menu pointer for later use
menu->AddItem(smenu);								// Add language menu to "Preference" menu
menu->SetTargetForItems(this);						// All "Language" Message should be send to the window
menubar->AddItem(menu);								// Add "Preferences" menu to menubar

AddChild(menubar);									// Add menubar to window

//----- Create backgound view ------------------------------------------------
frame = Bounds();
frame.top = menubar->Bounds().Height() + 1.0;
cBkgView = new HelloLocaleBkgView(frame);
AddChild(cBkgView);


rect = cBkgView->Bounds();
left = 10;
right = rect.right - left;
top = 5;

//----- Create some views (String and Button) for the window ------------------
strview = new BStringView(BRect((HL_WIDTH - BUTTON_DX)/2, HL_HEIGHT/3, (HL_WIDTH + BUTTON_DX)/2, HL_HEIGHT/3+20), NULL, cLang->Text(SFHELLOTEXT), B_FOLLOW_H_CENTER+B_FOLLOW_V_CENTER); 
cBkgView->AddChild(strview);

string = cLang->Text(SFQUIT);
string += "...";
button = new BButton(BRect((HL_WIDTH - BUTTON_DX)/2, rect.bottom - BUTTON_DY - 10, (HL_WIDTH + BUTTON_DX)/2, rect.bottom - 10),
					"btn_quit" , string.String(),
					new BMessage(B_QUIT_REQUESTED), B_FOLLOW_H_CENTER+B_FOLLOW_BOTTOM);
cBkgView->AddChild(button);

}

/*****************************************************************************
~HelloLocaleWin	Destructor; Free allocated data of the window instance. Will#
			be called by the OS.

Usage:		-

Parameter:	-

Returns:	-

Comment:	-
*****************************************************************************/
HelloLocaleWin::~HelloLocaleWin()
{
stop_watching(this);							// deactivating node watching
}

/*****************************************************************************
MessageReceived	Hook function; called by the OS if a message receive the 
			window.

Usage:		-

Parameter:	message		Pointer to BMessage

Returns:	-

Comments:	-
*****************************************************************************/
void HelloLocaleWin::MessageReceived(BMessage *msg)
{
entry_ref	ref;
BDirectory	dir;
BFile		file;
app_info	info; 
BPath		path; 
BEntry		entry;
BAlert		*alert;

switch (msg->what)
	{
	//---- Set language ----------------------------------------------------
	case MSG_SFLANGUAGE:
		be_app->GetAppInfo(&info); 								// get program info
		entry.SetTo(&info.ref); 								// get access to program
		entry.GetPath(&path); 									// path + filename
		path.GetParent(&path);									// only application path is needed
		path.Append("locale");									// append language folder
		cLang->SetLanguage(&path, msg->FindString("language"));
		cPref.ReplaceString("language", msg->FindString("language"));
		path.GetParent(&path);									// only application path is needed
		path.Append("HelloLocale.pref");						// Append filename for settings
		file.SetTo(path.Path(), B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
		if (file.InitCheck() == B_OK)
			{
			cPref.Flatten(&file);								// Save new Preferences
			alert = new BAlert("Locale", cLang->Text(SFRESTARTNEWLOCALE), cLang->Text(SFCONTINUE));
			alert->Go();
			}
		break;
	//---- Changes at the 'locale' folder ------------------------------------ 
	case B_NODE_MONITOR:
		UpdateLocaleMenu(msg);
		break;
	//-------- Default message management ------------------------------------
	default:
		BWindow::MessageReceived(msg);
	}
}

/*****************************************************************************
QuitRequested	Hook function; called by the OS if a "B_QUIT_REQUESTED" 
			message was sent to the window.

Usage:		-

Parameter:	-

Returns:	true	Window can be closed
			false	Don't close the window

Comment:	-
*****************************************************************************/
bool HelloLocaleWin::QuitRequested()
{
be_app->PostMessage(B_QUIT_REQUESTED);

return(true);
}
/*****************************************************************************
BuildLanguageMenu	Create a language menu.

Usage:		BMenu		*menu = new BMenu(...);
			const char	*def = "English";
			menu = BuildlanguageMenu(menu, def);

Parameter:	menu		Pointer to an BMenu where the item should be added
			def			Pointer to the language that should be marked

Returns:	-

Comment:	-
*****************************************************************************/
void HelloLocaleWin::BuildLanguageMenu(BMenu *menu, const char *def)
{
BMenuItem	*item;
app_info	info; 
BPath		path; 
BEntry		entry;
BNode		node;
BDirectory	*dir;
BString		name;
int32		i;
char		buf[64];
ssize_t		len;

be_app->GetAppInfo(&info); 								// get program info
entry.SetTo(&info.ref); 								// get access to program
entry.GetPath(&path); 									// path + filename
path.GetParent(&path);									// only application path is needed
path.Append("locale");									// append language folder
dir = new BDirectory(path.Path());						// Zugriff auf Sprachdaten-Verzeichnis

item = CreateLanguageItem(SF_DEFAULT_LANGUAGE);			// create default language menu item
InsertLanguageItem(menu, item);							// insert item to menu

while(dir->GetNextEntry(&entry, TRUE) == B_OK)			// Check the folder
	{			
	if (entry.InitCheck() != B_NO_ERROR)				// problems?
		break;											// if so -> bye bye
	node.SetTo(&entry);
	if ((len = node.ReadAttr("sf:language", B_STRING_TYPE, 0, buf, 64)) > 0)
		{
		buf[len] = '\0';
		item = CreateLanguageItem(buf);					// create item
		InsertLanguageItem(menu, item);					// Add item to menu
		}
	}
menu->SetRadioMode(true);								// only one item is to select
menu->SetTargetForItems(this);							// All Message should be send to the window

i = 0;
while ((item = menu->ItemAt(i++)) != NULL)				// loop the menu list
	{
	if (strcmp(item->Label(), def) == 0)				// search the default name
		{
		item->SetMarked(true);							// mark the default
		break;
		}
	}
delete dir;
}

/*****************************************************************************
InsertLanguageItem	Add item to language menu.

Usage:		BMenu		*menu;
			BMenuItem	*item = CreateLanguageItem(...);
			InsertLanguageItem(menu, item);

Parameter:	menu	Pointer to language
			item	Item to be added 
					
Returns:	-

Comment:	Insert the items alphabetically.
*****************************************************************************/
void HelloLocaleWin::InsertLanguageItem(BMenu *menu, BMenuItem *newitem)
{
int32		count = menu->CountItems();
int32		index, flag;
BMenuItem	*item;
BString		string;

for (index = 0; index < count; index++)
	{
	item = menu->ItemAt(index);
	flag = strcmp(newitem->Label(), item->Label());
	if (flag < 0)
		break;
	else if (flag == 0)
		{
		index++;
		string = newitem->Label();
		string += "(2)";
		newitem->SetLabel(string.String());
		break;
		}
	}
menu->AddItem(newitem, index);
}

/*****************************************************************************
CreateLanguageItem	Create a new language menu item.

Usage:		const char	*name;
			BMenuItem	*item;
			item = CreateLanguageItem(name);

Parameter:	name	Name of the language for the item
					
Returns:	item	Pointer to created item

Comment:	-
*****************************************************************************/
BMenuItem *HelloLocaleWin::CreateLanguageItem(const char *name)
{
BMenuItem	*item;
BMessage	*msg;

msg = new BMessage(MSG_SFLANGUAGE);						// Create Message
msg->AddString("language", name);						// Add filename
item = new BMenuItem(name, msg);						// Create menu item

return (item);
}


/*****************************************************************************
UpdateLocaleMenu	The contents of the "locale" folder has changed. 
			Synchronize the language menu with the folder.

Usage:		BMessage	*msg;
			UpdateLocaleMenu(msg);

Parameter:	msg		Pointer to the message of the node monitor. 
					Message with the type 'B_NODE_MONITOR'
					
Returns:	-

Comment:	-
*****************************************************************************/
void HelloLocaleWin::UpdateLocaleMenu(BMessage *msg)
{
BMenuItem	*item;
BString		string;
int32		opcode;

if (msg->FindInt32("opcode", &opcode) == B_OK)
	{ 
	switch (opcode)
		{ 
		//---- file/folder deleted ------------------------------------------- 
		case B_ENTRY_REMOVED:
		//---- file/folder moved --------------------------------------------- 
		case B_ENTRY_MOVED:
		//---- new file/folder created --------------------------------------- 
		case B_ENTRY_CREATED:
			if (cLocaleMenu)
				{
				while ((item = cLocaleMenu->RemoveItem((int32)0)) > (BMenuItem *)NULL)
					{
					if (item->IsMarked())
						string = item->Label();					
					delete item;
					}
				BuildLanguageMenu(cLocaleMenu, string.String());
				}
			break; 
		}
	}
}


/*****************************************************************************
Routins for the background view
*****************************************************************************/

/*****************************************************************************
HelloLocaleBkgView	Contructor; Create a background view for a window.

Usage:		HelloLocaleBkgView	*view;
			BRect		frame = Bounds();
			view = new HelloLocaleBkgView(frame);
			
Parameter:	frame		Position and Size of the view. As coordinate system
						the local system of the window is used.

Returns:	-

Comment:	-
*****************************************************************************/
HelloLocaleBkgView::HelloLocaleBkgView(BRect rect)
				: BView(rect, "bkg_view", B_FOLLOW_ALL_SIDES, B_NAVIGABLE | B_WILL_DRAW | B_FRAME_EVENTS)
{

}

/*****************************************************************************
AttachedToWindow	Hook function; called by the OS, when the view is added 
			to the window. Used to set some defaults (font, color) for
			the view.
			
Aufruf:		-
			
Paramter:	-

Returns:	-

Bemerkung:	-
*****************************************************************************/
void HelloLocaleBkgView::AttachedToWindow()
{
//SetViewColor(BeButtonGrey);						// set view color
SetFont(be_bold_font);							// set font
SetFontSize(12);								// set font height
BView::AttachedToWindow();
}
